-- The concrete syntax for Italian of the abstract language Food

concrete FoodIta of Food = open Prelude in {
  lincat
    -- interpretation of the atomic types
    -- a lincat for each 'cat'
    Comment = {s : Str } ;
    Quality = Adjective ;
    Kind = Noun ;
    Item = NounPhrase ;

    -- interpretation of the abstract constants
    -- a lin for each 'fun'
  lin
    Pred item quality =
      {s = item.s ++ copula ! item.n ++ quality.s ! item.g ! item.n } ;
    This = det Sg "questo" "questa";
    That = det Sg "quel" "quella";
    These = det Pl "questi" "queste";
    Those = det Pl "quei" "quelle";
    Mod quality kind =
      {
	s = \\n => kind.s ! n ++ quality.s ! kind.g ! n ;
	g = kind.g
      } ;
    Wine = noun "vino" "vini" Masc ;
    Cheese = noun "formaggio" "formaggi" Masc ;
    Fish = noun "pesce" "pesci" Masc ;
    Very quality = {s = \\g,n => "molto" ++ quality.s ! g ! n} ;
    Fresh = adjective "fresco" "fresca" "freschi" "fresche" ;
    Warm = regAdj "caldo" ;
    Italian = regAdj "italiano" ;
    Expensive = regAdj "caro" ;
    Delicious = regAdj "delizioso" ;
    Boring = regAdj "noioso" ;

  param
    Number = Sg | Pl ;
    Gender = Masc | Fem ;

  oper
    NounPhrase : Type =
      {s : Str ; g : Gender ; n : Number} ;
    Noun : Type = {s : Number => Str ; g : Gender};
    Adjective : Type = {s : Gender => Number => Str} ;

    det : Number -> Str -> Str -> Noun -> NounPhrase =
      \n,m,f,cn -> {
	s = table {Masc => m ; Fem => f} ! cn.g ++ cn.s ! n ;
	g = cn.g ;
	n = n 
	} ;
    noun : Str -> Str -> Gender -> Noun =
      \vino,vini,g -> {
	s = table {Sg => vino ; Pl => vini} ;
	g = g
      } ;
    adjective :
      (nero,nera,neri,nere : Str) -> Adjective =
      \nero,nera,neri,nere -> {
	s = table {
	  Masc => table {Sg => nero ; Pl => neri} ;
	  Fem => table {Sg => nera ; Pl => nere}
	  }
      } ;
    regAdj : Str -> Adjective =
      \nero ->
      let ner : Str = init nero in
      adjective nero (ner+"a") (ner+"i") (ner+"e") ;
	    
    copula : Number => Str =
      table {Sg => "è" ; Pl => "sono"} ;

}
